
from logging import Logger
from logging import getLogger
from typing import cast
from xml.dom.minidom import Document
from xml.dom.minidom import Element

from ogl.OglActor import OglActor
from ogl.OglUseCase import OglUseCase
from pyutmodel.PyutActor import PyutActor
from pyutmodel.PyutUseCase import PyutUseCase

from oglio.Types import OglActors
from oglio.Types import OglUseCases
from oglio.toXmlV10.BaseOglToDom import BaseOglToDom
from oglio.toXmlV10.PyutToDom import PyutToDom
from oglio.toXmlV10.XmlConstants import XmlConstants


class OglUseCasesToDom(BaseOglToDom):
    def __init__(self,  xmlDocument: Document):

        super().__init__(xmlDocument)

        self.useCaseLogger: Logger = getLogger(__name__)

        self._pyutToMiniDom: PyutToDom = PyutToDom()

    def serialize(self, documentNode: Element, oglUseCases: OglUseCases, oglActors: OglActors) -> Element:
        """

        Args:
            documentNode:
            oglUseCases:    Ogl Use Case objects
            oglActors:      Ogl Actor objects

        Returns:  The minidom element
        """

        for oglActor in oglActors:
            actorElement: Element = self._oglActorToDom(oglActor=oglActor, xmlDoc=self._xmlDocument)
            documentNode.appendChild(actorElement)

        for oglUseCase in oglUseCases:
            self.useCaseLogger.debug(f'{oglUseCase}')
            useCaseElement: Element = self._oglUseCaseToDom(oglUseCase=oglUseCase, xmlDoc=self._xmlDocument)
            documentNode.appendChild(useCaseElement)

        return documentNode

    def _oglUseCaseToDom(self, oglUseCase: OglUseCase, xmlDoc: Document) -> Element:
        """
        Export an OglUseCase to a minidom Element.

        Args:
            oglUseCase:  UseCase to convert
            xmlDoc:      xml document

        Returns:
            A new use case minidom element
        """
        useCaseElement: Element = xmlDoc.createElement(XmlConstants.ELEMENT_GRAPHIC_USE_CASE)

        self._appendOglBase(oglUseCase, useCaseElement)

        useCaseElement.appendChild(self._pyutToMiniDom.pyutUseCaseToDom(cast(PyutUseCase, oglUseCase.pyutObject), xmlDoc))

        return useCaseElement

    def _oglActorToDom(self, oglActor: OglActor, xmlDoc: Document) -> Element:
        """
        Exporting an OglActor to a minidom Element.

        Args:
            oglActor:   Actor to convert
            xmlDoc:     xml document

        Returns:
            New minidom element
        """
        pyutActorElement: Element = xmlDoc.createElement(XmlConstants.ELEMENT_GRAPHIC_ACTOR)

        self._appendOglBase(oglActor, pyutActorElement)

        pyutActorElement.appendChild(self._pyutToMiniDom.pyutActorToDom(cast(PyutActor, oglActor.pyutObject), xmlDoc))

        return pyutActorElement




from typing import cast

from logging import Logger
from logging import getLogger

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from oglio.Types import OglTexts

from ogl.OglText import OglText

from oglio.toXmlV11.PyutToXml import PyutToXml
from oglio.toXmlV11.XmlConstants import XmlConstants
from oglio.toXmlV11.BaseOglToXml import BaseOglToXml


class OglTextsToXml(BaseOglToXml):
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._pyutToXml: PyutToXml = PyutToXml()

    def serialize(self, documentTop: Element, oglTexts: OglTexts) -> Element:

        for text in oglTexts:
            oglText: OglText = cast(OglText, text)
            oglTextElement: Element = self._oglTextToXml(documentTop=documentTop, oglText=oglText)
            self._pyutToXml.pyutTextToXml(pyutText=oglText.pyutObject, oglTextElement=oglTextElement)

        return documentTop

    def _oglTextToXml(self, documentTop: Element, oglText: OglText) -> Element:

        attributes = self._oglBaseAttributes(oglObject=oglText)
        oglTextSubElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_TEXT, attrib=attributes)

        return oglTextSubElement




from logging import Logger
from logging import getLogger

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import ElementTree
from xml.etree.ElementTree import indent
from xml.etree.ElementTree import SubElement
from xml.etree.ElementTree import tostring
from xml.etree.ElementTree import fromstring

from oglio.Types import OglDocument

from oglio.toXmlV11.OglClassToXml import OglClassToXml
from oglio.toXmlV11.OglLinksToXml import OglLinksToXml
from oglio.toXmlV11.OglNotesToXml import OglNotesToXml
from oglio.toXmlV11.OglSequenceToXml import OglSequenceToXml
from oglio.toXmlV11.OglTextsToXml import OglTextsToXml
from oglio.toXmlV11.OglUseCasesToXml import OglUseCasesToXml

from oglio.toXmlV11.XmlConstants import XmlConstants

XML_VERSION: str = '11'

INDENT_SPACES: str = '    '     # TODO: Make this configurable
PRETTY_PRINT:  bool = True


class OglToXml:
    def __init__(self, projectCodePath: str):

        self.logger: Logger = getLogger(__name__)

        self._projectCodePath: str     = projectCodePath
        topElement:            Element = Element(XmlConstants.TOP_LEVEL_ELEMENT)

        topElement.set(XmlConstants.ATTR_VERSION, XML_VERSION)
        topElement.set(XmlConstants.ATTR_CODE_PATH, projectCodePath)

        tree: ElementTree = ElementTree(topElement)
        indent(tree, space='    ')

        self._topElement:  Element     = topElement
        self._tree:        ElementTree = tree
        self._prettyPrint: bool        = PRETTY_PRINT

    @property
    def prettyPrint(self) -> bool:
        return self._prettyPrint

    @prettyPrint.setter
    def prettyPrint(self, newValue: bool):
        self._prettyPrint = newValue

    @property
    def xml(self) -> str:
        """

        Returns:  The serialized XML in pretty print format
        """
        if self.prettyPrint is True:
            return self._toPrettyString(self._topElement)
        else:
            return self._toString(self._topElement)

    def serialize(self, oglDocument: OglDocument):

        oglClassToXml:    OglClassToXml    = OglClassToXml()
        oglLinksToXml:    OglLinksToXml    = OglLinksToXml()
        oglNotesToXml:    OglNotesToXml    = OglNotesToXml()
        oglTextsToXml:    OglTextsToXml    = OglTextsToXml()
        oglUseCasesToXml: OglUseCasesToXml = OglUseCasesToXml()
        oglSequenceToXml: OglSequenceToXml = OglSequenceToXml()

        documentElement: Element = self._oglDocumentToXml(oglDocument=oglDocument)

        oglClassToXml.serialize(documentTop=documentElement, oglClasses=oglDocument.oglClasses)
        oglLinksToXml.serialize(documentTop=documentElement, oglLinks=oglDocument.oglLinks)
        oglNotesToXml.serialize(documentTop=documentElement, oglNotes=oglDocument.oglNotes)
        oglTextsToXml.serialize(documentTop=documentElement, oglTexts=oglDocument.oglTexts)

        oglUseCasesToXml.serialize(documentTop=documentElement, oglUseCases=oglDocument.oglUseCases, oglActors=oglDocument.oglActors)
        oglSequenceToXml.serialize(documentTop=documentElement, oglSDInstances=oglDocument.oglSDInstances, oglSDMessages=oglDocument.oglSDMessages)

    def writeXml(self, fqFileName):
        """
        Persist the XML

        Args:
            fqFileName:  The fully qualified file name
        """
        with open(fqFileName, 'w') as fd:
            fd.write(self.xml)

    def _oglDocumentToXml(self, oglDocument: OglDocument) -> Element:

        attributes = {
            XmlConstants.ATTR_TYPE:              oglDocument.documentType,
            XmlConstants.ATTR_TITLE:             oglDocument.documentTitle,
            XmlConstants.ATTR_SCROLL_POSITION_X: str(oglDocument.scrollPositionX),
            XmlConstants.ATTR_SCROLL_POSITION_Y: str(oglDocument.scrollPositionY),
            XmlConstants.ATTR_PIXELS_PER_UNIT_X: str(oglDocument.pixelsPerUnitX),
            XmlConstants.ATTR_PIXELS_PER_UNIT_Y: str(oglDocument.pixelsPerUnitY),
        }
        documentTop: Element = SubElement(self._topElement, XmlConstants.ELEMENT_DOCUMENT, attrib=attributes)

        return documentTop

    def _toPrettyString(self, originalElement: Element):
        """
        Create a copy of the input originalElement
        Convert to string, then parse again
        tostring() returns a binary, so we need to decode it to get a string

        Args:
            originalElement:

        Returns:  Pretty printed XML
        """
        elementCopy: Element = fromstring(tostring(originalElement))
        indent(elementCopy, space=INDENT_SPACES, level=0)

        return self._toString(elementCopy)

    def _toString(self, element: Element) -> str:
        return tostring(element, encoding='iso-8859-1', xml_declaration=True).decode('utf-8')




from logging import Logger
from logging import getLogger

from oglio.IDFactory import IDFactory


class BaseXml:
    def __init__(self):
        self.logger: Logger = getLogger(__name__)

        self._idFactory: IDFactory = IDFactory()





from logging import Logger
from logging import getLogger
from typing import cast

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from ogl.OglActor import OglActor
from ogl.OglUseCase import OglUseCase

from oglio.Types import OglActors
from oglio.Types import OglUseCases
from oglio.toXmlV11.XmlConstants import XmlConstants

from oglio.toXmlV11.BaseOglToXml import BaseOglToXml
from oglio.toXmlV11.PyutToXml import PyutToXml


class OglUseCasesToXml(BaseOglToXml):
    """
    TODO:  Don't forget the OGL Actors !!

    """
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._pyutToXml: PyutToXml = PyutToXml()

    def serialize(self, documentTop: Element, oglUseCases: OglUseCases, oglActors: OglActors) -> Element:

        for actor in oglActors:
            oglActor:        OglActor = cast(OglActor, actor)
            oglActorElement: Element  = self._oglActorToXml(documentTop=documentTop, oglActor=oglActor)
            self._pyutToXml.pyutActorToXml(pyutActor=oglActor.pyutObject, oglActorElement=oglActorElement)

        for useCase in oglUseCases:
            oglUseCase:        OglUseCase = cast(OglUseCase, useCase)
            oglUseCaseElement: Element    = self._oglUseCaseToXml(documentTop=documentTop, oglUseCase=oglUseCase)
            self._pyutToXml.pyutUseCaseToXml(pyutUseCase=oglUseCase.pyutObject, oglUseCaseElement=oglUseCaseElement)

        return documentTop

    def _oglUseCaseToXml(self, documentTop: Element, oglUseCase: OglUseCase) -> Element:

        attributes = self._oglBaseAttributes(oglObject=oglUseCase)
        oglTextSubElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_USE_CASE, attrib=attributes)

        return oglTextSubElement

    def _oglActorToXml(self, documentTop: Element, oglActor: OglActor) -> Element:

        attributes = self._oglBaseAttributes(oglObject=oglActor)
        pyutActorSubElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_ACTOR, attrib=attributes)

        return pyutActorSubElement




class XmlConstants:
    """
    A `no method` class that just hosts the strings that represent the Pyut XML strings
    """

    TOP_LEVEL_ELEMENT:    str = 'PyutProject'
    ELEMENT_OGL_CLASS:    str = 'OglClass'
    ELEMENT_OGL_LINK:     str = 'OglLink'
    ELEMENT_OGL_NOTE:     str = 'OglNote'
    ELEMENT_OGL_TEXT:     str = 'OglText'
    ELEMENT_OGL_USE_CASE: str = 'OglUseCase'
    ELEMENT_OGL_ACTOR:    str = 'OglActor'

    ELEMENT_OGL_SD_INSTANCE: str = 'OglSDInstance'
    ELEMENT_OGL_SD_MESSAGE:  str = 'OglSDMessage'

    ELEMENT_OGL_INTERFACE2:    str = 'OglInterface2'

    ELEMENT_DOCUMENT:             str = 'PyutDocument'
    ELEMENT_PYUT_CLASS:           str = 'PyutClass'
    ELEMENT_PYUT_METHOD:          str = 'PyutMethod'
    ELEMENT_MODEL_PYUT_PARAMETER: str = 'PyutParameter'
    ELEMENT_MODEL_PYUT_FIELD:     str = 'PyutField'
    ELEMENT_PYUT_LINK:            str = 'PyutLink'
    ELEMENT_PYUT_NOTE:            str = 'PyutNote'

    ELEMENT_PYUT_USE_CASE:        str = 'PyutUseCase'
    ELEMENT_PYUT_TEXT:            str = 'PyutText'
    ELEMENT_PYUT_ACTOR:           str = 'PyutActor'

    ELEMENT_PYUT_SD_INSTANCE:   str = 'PyutSDInstance'
    ELEMENT_PYUT_SD_MESSAGE:    str = 'PyutSDMessage'

    ELEMENT_MODEL_INTERFACE:      str = 'PyutInterface'
    ELEMENT_IMPLEMENTOR:          str = 'Implementor'

    ELEMENT_MODEL_MODIFIER:    str = 'Modifier'
    ELEMENT_MODEL_SOURCE_CODE: str = 'SourceCode'
    ELEMENT_MODEL_CODE:        str = 'Code'

    ELEMENT_MODEL_CONTROL_POINT: str = 'ControlPoint'

    ELEMENT_ASSOCIATION_CENTER_LABEL:      str = 'LabelCenter'
    ELEMENT_ASSOCIATION_SOURCE_LABEL:      str = 'LabelSource'
    ELEMENT_ASSOCIATION_DESTINATION_LABEL: str = 'LabelDestination'

    ATTR_VERSION: str = 'version'

    ATTR_ID: str = 'id'

    ATTR_WIDTH:  str = 'width'
    ATTR_HEIGHT: str = 'height'

    ATTR_X: str = 'x'
    ATTR_Y: str = 'y'

    ATTR_STEREOTYPE:  str = 'stereotype'
    ATTR_DESCRIPTION: str = 'description'
    ATTR_VISIBILITY:  str = 'visibility'

    ATTR_FILENAME: str = 'fileName'
    ATTR_NAME:     str = 'name'
    ATTR_CONTENT:  str = 'content'
    ATTR_TYPE:     str = 'type'
    ATTR_TITLE:    str = 'title'

    ATTR_DEFAULT_VALUE:      str = 'defaultValue'
    ATTR_DISPLAY_STEREOTYPE: str = 'displayStereotype'
    ATTR_DISPLAY_METHODS:    str = 'displayMethods'
    ATTR_DISPLAY_FIELDS:     str = 'displayFields'
    ATTR_DISPLAY_PARAMETERS: str = 'displayParameters'
    ATTR_FILE_NAME:          str = 'fileName'

    ATTR_METHOD_RETURN_TYPE: str = 'returnType'

    ATTR_SPLINE:               str = 'spline'
    ATTR_LINK_SOURCE_ANCHOR_X: str = 'sourceAnchorX'
    ATTR_LINK_SOURCE_ANCHOR_Y: str = 'sourceAnchorY'

    ATTR_LINK_DESTINATION_ANCHOR_X: str = 'destinationAnchorX'
    ATTR_LINK_DESTINATION_ANCHOR_Y: str = 'destinationAnchorY'

    ATTR_BIDIRECTIONAL: str = 'bidirectional'

    ATTR_SOURCE_ID:      str = 'sourceId'
    ATTR_DESTINATION_ID: str = 'destinationId'

    ATTR_CARDINALITY_SOURCE:      str = 'cardinalitySource'
    ATTR_CARDINALITY_DESTINATION: str = 'cardinalityDestination'

    ATTR_INSTANCE_NAME:    str = 'instanceName'
    ATTR_LIFE_LINE_LENGTH: str = 'lifeLineLength'

    ATTR_MESSAGE:          str = 'message'
    ATTR_SOURCE_TIME:      str = 'sourceTime'
    ATTR_DESTINATION_TIME: str = 'destinationTime'

    ATTR_SD_MESSAGE_SOURCE_ID:      str = 'sourceID'
    ATTR_SD_MESSAGE_DESTINATION_ID: str = 'destinationID'

    ATTR_CODE_PATH: str = 'CodePath'

    ATTR_SCROLL_POSITION_X: str = 'scrollPositionX'
    ATTR_SCROLL_POSITION_Y: str = 'scrollPositionY'
    ATTR_PIXELS_PER_UNIT_X: str = 'pixelsPerUnitX'
    ATTR_PIXELS_PER_UNIT_Y: str = 'pixelsPerUnitY'

    ATTR_LOLLIPOP_ATTACHMENT_POINT: str = 'attachmentPoint'
    ATTR_IMPLEMENTING_CLASS_NAME:   str = 'implementingClassName'

    ATTR_TEXT_SIZE:     str = 'textSize'
    ATTR_IS_BOLD:       str = 'isBold'
    ATTR_IS_ITALICIZED: str = 'isItalicized'
    ATTR_FONT_FAMILY:   str = 'fontFamily'




from typing import cast

from logging import Logger
from logging import getLogger

from zlib import decompress
from zlib import ZLIB_VERSION

from untanglepyut.Types import Documents
from untanglepyut.Types import ProjectInformation

from untanglepyut.UnTangleProjectInformation import UnTangleProjectInformation

from untanglepyut.UnTangler import UnTangler
from untanglepyut.XmlVersion import XmlVersion

from oglio.Types import OglActors
from oglio.Types import OglClasses
from oglio.Types import OglDocument
from oglio.Types import OglLinks
from oglio.Types import OglNotes
from oglio.Types import OglProject
from oglio.Types import OglTexts
from oglio.Types import OglUseCases
from oglio.Types import OglSDInstances
from oglio.Types import OglSDMessages

from oglio.UnsupportedFileTypeException import UnsupportedFileTypeException
from oglio.UnsupportedVersion import UnsupportedVersion


class Reader:
    """
    This is a simple translation layer on top of the PyutUntangler library.  This
    layer simply hides that implementation detail and provides a more usable
    interface to Pyut.  Additionally, it serves as a common piece of code
    that allows and IOPlugin implementations
    See https://github.com/hasii2011/pyutplugincore
    """
    def __init__(self):

        self.logger: Logger = getLogger(__name__)

    def readFile(self, fqFileName: str) -> OglProject:
        """
        Parse the input .put file

        Args:
            fqFileName: The fully qualified file name
        """
        if fqFileName.endswith('.put') is False:
            raise UnsupportedFileTypeException(message=f'File does not end with .put suffix')

        rawXmlString:       str                = self._decompressFile(fqFileName=fqFileName)
        projectInformation: ProjectInformation = self._extractProjectInformation(fqFileName)

        if projectInformation.version == XmlVersion.V10.value:
            untangler: UnTangler = UnTangler(xmlVersion=XmlVersion.V10)
        else:
            untangler = UnTangler(xmlVersion=XmlVersion.V11)

        untangler.untangleXml(xmlString=rawXmlString, fqFileName=fqFileName)

        oglProject: OglProject = self._makeOglProject(untangler=untangler)

        return oglProject

    def readXmlFile(self, fqFileName: str) -> OglProject:
        """
        Parse the input XML file;

        Args:
            fqFileName: Fully qualified file name
        """
        if fqFileName.endswith('.xml') is False:
            raise UnsupportedFileTypeException(message=f'File does not end with .xml suffix')

        projectInformation: ProjectInformation = self._extractProjectInformation(fqFileName)

        if projectInformation.version == '10':
            untangler: UnTangler = UnTangler(xmlVersion=XmlVersion.V10)
        elif projectInformation.version == '11':
            untangler = UnTangler(xmlVersion=XmlVersion.V11)
        else:
            raise UnsupportedVersion(message=f'Unsupported version: {projectInformation.version}')

        untangler.untangleFile(fqFileName=fqFileName)

        oglProject: OglProject = self._makeOglProject(untangler=untangler)

        return oglProject

    def _decompressFile(self, fqFileName: str) -> str:
        """
        Decompresses a previously Pyut compressed file
        Args:
            fqFileName: Fully qualified file name with a .put suffix

        Returns:  A raw XML String
        """
        try:
            with open(fqFileName, "rb") as compressedFile:
                compressedData: bytes = compressedFile.read()
        except (ValueError, Exception) as e:
            self.logger.error(f'decompress open:  {e}')
            raise e
        else:
            self.logger.info(f'{ZLIB_VERSION=}')
            xmlBytes:  bytes = decompress(compressedData)  # has b'....' around it
            xmlString: str   = xmlBytes.decode()
            self.logger.debug(f'Document read:\n{xmlString}')

        return xmlString

    def _makeOglProject(self, untangler: UnTangler) -> OglProject:
        """
        Syntactic sugar

        Args:
            untangler:

        Returns:  A populated  OglProject
        """
        oglProject: OglProject = OglProject()

        oglProject.toOglProject(untangler.projectInformation)

        assert oglProject.version == XmlVersion.V10.value or oglProject.version == XmlVersion.V11.value, 'We have mismatched XML versions'
        documents: Documents = untangler.documents
        for document in documents.values():
            self.logger.debug(f'Untangled - {document.documentTitle}')
            oglDocument: OglDocument = OglDocument()
            oglDocument.toOglDocument(document)
            #
            # Cheat by just type casting
            #
            oglDocument.oglClasses  = cast(OglClasses,  document.oglClasses)
            oglDocument.oglLinks    = cast(OglLinks,    document.oglLinks)
            oglDocument.oglNotes    = cast(OglNotes,    document.oglNotes)
            oglDocument.oglTexts    = cast(OglTexts,    document.oglTexts)
            oglDocument.oglActors   = cast(OglActors,   document.oglActors)
            oglDocument.oglUseCases = cast(OglUseCases, document.oglUseCases)
            oglDocument.oglSDInstances = cast(OglSDInstances, document.oglSDInstances)
            oglDocument.oglSDMessages  = cast(OglSDMessages,  document.oglSDMessages)

            self.logger.debug(f'OglDocument - {oglDocument}')
            oglProject.oglDocuments[oglDocument.documentTitle] = oglDocument

        return oglProject

    def _extractProjectInformation(self, fqFileName: str) -> ProjectInformation:

        unTangleProjectInformation: UnTangleProjectInformation = UnTangleProjectInformation(fqFileName=fqFileName)

        return unTangleProjectInformation.projectInformation




from logging import Logger
from logging import getLogger

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from ogl.sd.OglSDInstance import OglSDInstance
from ogl.sd.OglSDMessage import OglSDMessage
from pyutmodel.PyutSDMessage import PyutSDMessage

from oglio.Types import OglSDInstances
from oglio.Types import OglSDMessages
from oglio.toXmlV11.InternalTypes import ElementAttributes

from oglio.toXmlV11.XmlConstants import XmlConstants
from oglio.toXmlV11.BaseOglToXml import BaseOglToXml
from oglio.toXmlV11.PyutToXml import PyutToXml


class OglSequenceToXml(BaseOglToXml):
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._pyutToXml: PyutToXml = PyutToXml()

    def serialize(self, documentTop: Element, oglSDMessages: OglSDMessages, oglSDInstances: OglSDInstances) -> Element:

        for oglSDInstance in oglSDInstances.values():
            self._oglSDInstanceToXml(documentTop=documentTop, oglSDInstance=oglSDInstance)

        for oglSDMessage in oglSDMessages.values():
            self._oglSDMessageToXml(documentTop=documentTop, oglSDMessage=oglSDMessage)

        return documentTop

    def _oglSDInstanceToXml(self,  documentTop: Element, oglSDInstance: OglSDInstance,) -> Element:
        """
        Export an OglSDInstance to a minidom Element

        Args:
            documentTop:    XML Element
            oglSDInstance:  Instance to convert

        Returns:
            An element
        """
        attributes: ElementAttributes = self._oglBaseAttributes(oglObject=oglSDInstance)
        oglSDInstanceElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_SD_INSTANCE, attrib=attributes)

        self._pyutToXml.pyutSDInstanceToXml(pyutSDInstance=oglSDInstance.pyutObject, oglSDInstanceElement=oglSDInstanceElement)

        return oglSDInstanceElement

    def _oglSDMessageToXml(self, documentTop: Element, oglSDMessage: OglSDMessage) -> Element:
        """
        Export an OglSDMessage to a minidom Element.

        Args:
            documentTop:    XML Element
            oglSDMessage:   Message to convert

        Returns:
            An element
        """
        pyutSDMessage: PyutSDMessage = oglSDMessage.pyutSDMessage

        oglSDMessageElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_SD_MESSAGE)

        self._pyutToXml.pyutSDMessageToXml(pyutSDMessage=pyutSDMessage, oglSDMessageElement=oglSDMessageElement)
        return oglSDMessageElement
